@@ -23,14 +23,26 @@ import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
 import java.io.IOException;
+import java.io.Reader;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 
 import org.apache.commons.lang.SystemUtils;
 import org.apache.maven.plugin.MojoExecutionException;
+import org.apache.maven.plugin.javadoc.ProxyServer.AuthAsyncProxyServlet;
 import org.apache.maven.plugin.testing.AbstractMojoTestCase;
+import org.apache.maven.settings.Proxy;
+import org.apache.maven.settings.Settings;
 import org.codehaus.plexus.util.FileUtils;
+import org.codehaus.plexus.util.IOUtil;
+import org.codehaus.plexus.util.ReaderFactory;
 import org.codehaus.plexus.util.StringUtils;
 
 /**
+ * Test {@link org.apache.maven.plugin.javadoc.JavadocReport} class.
+ *
  * @author <a href="mailto:oching@apache.org">Maria Odea Ching</a>
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
  */
@@ -39,24 +51,16 @@ public class JavadocReportTest
 {
     private static final String LINE_SEPARATOR = " ";
 
-    /**
-     * @see junit.framework.TestCase#setUp()
-     */
+    /** flag to copy repo only one time */
+    private static boolean TEST_REPO_CREATED = false;
+
+    /** {@inheritDoc} */
     protected void setUp()
         throws Exception
     {
-        // required for mojo lookups to work
         super.setUp();
-        createTestRepo();
-    }
 
-    /**
-     * @see junit.framework.TestCase#tearDown()
-     */
-    protected void tearDown()
-        throws Exception
-    {
-        super.tearDown();
+        createTestRepo();
     }
 
     /**
@@ -67,81 +71,74 @@ public class JavadocReportTest
     private void createTestRepo()
         throws IOException
     {
+        if ( TEST_REPO_CREATED )
+        {
+            return;
+        }
+
+        File localRepo = new File( getBasedir(), "target/local-repo/" );
+        localRepo.mkdirs();
+
         // ----------------------------------------------------------------------
         // UMLGraph
         // ----------------------------------------------------------------------
 
-        File f = new File( getBasedir(), "target/local-repo/umlgraph/UMLGraph/2.1" );
-        f.mkdirs();
-
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/doclet-test/artifact-doclet/umlgraph/UMLGraph/maven-metadata-local.xml" ),
-                       new File( getBasedir(), "target/local-repo/umlgraph/UMLGraph/maven-metadata-local.xml" ) );
-
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/doclet-test/artifact-doclet/umlgraph/UMLGraph/2.1/UMLGraph-2.1.jar" ),
-                       new File( getBasedir(), "target/local-repo/umlgraph/UMLGraph/2.1/UMLGraph-2.1.jar" ) );
-
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/doclet-test/artifact-doclet/umlgraph/UMLGraph/2.1/UMLGraph-2.1.pom" ),
-                       new File( getBasedir(), "target/local-repo/umlgraph/UMLGraph/2.1/UMLGraph-2.1.pom" ) );
+        File sourceDir = new File( getBasedir(), "src/test/resources/unit/doclet-test/artifact-doclet" );
+        assertTrue( sourceDir.exists() );
+        FileUtils.copyDirectoryStructure( sourceDir, localRepo );
 
         // ----------------------------------------------------------------------
         // UMLGraph-bis
         // ----------------------------------------------------------------------
 
-        f = new File( getBasedir(), "target/local-repo/umlgraph/UMLGraph-bis/2.1" );
-        f.mkdirs();
-
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/doclet-path-test/artifact-doclet/umlgraph/UMLGraph-bis/maven-metadata-local.xml" ),
-                       new File( getBasedir(), "target/local-repo/umlgraph/UMLGraph-bis/maven-metadata-local.xml" ) );
-
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/doclet-path-test/artifact-doclet/umlgraph/UMLGraph-bis/2.1/UMLGraph-bis-2.1.jar" ),
-                       new File( getBasedir(), "target/local-repo/umlgraph/UMLGraph-bis/2.1/UMLGraph-bis-2.1.jar" ) );
-
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/doclet-path-test/artifact-doclet/umlgraph/UMLGraph-bis/2.1/UMLGraph-bis-2.1.pom" ),
-                       new File( getBasedir(), "target/local-repo/umlgraph/UMLGraph-bis/2.1/UMLGraph-bis-2.1.pom" ) );
+        sourceDir = new File( getBasedir(), "src/test/resources/unit/doclet-path-test/artifact-doclet" );
+        assertTrue( sourceDir.exists() );
+        FileUtils.copyDirectoryStructure( sourceDir, localRepo );
 
         // ----------------------------------------------------------------------
         // commons-attributes-compiler
         // http://www.tullmann.org/pat/taglets/
         // ----------------------------------------------------------------------
 
-        f = new File( getBasedir(), "target/local-repo/org/tullmann/taglets/1.0" );
-        f.mkdirs();
+        sourceDir = new File( getBasedir(), "src/test/resources/unit/taglet-test/artifact-taglet" );
+        assertTrue( sourceDir.exists() );
+        FileUtils.copyDirectoryStructure( sourceDir, localRepo );
 
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/taglet-test/artifact-taglet/org/tullmann/taglets/maven-metadata-local.xml" ),
-                       new File( getBasedir(), "target/local-repo/org/tullmann/taglets/maven-metadata-local.xml" ) );
+        // ----------------------------------------------------------------------
+        // stylesheetfile-test
+        // ----------------------------------------------------------------------
 
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/taglet-test/artifact-taglet/org/tullmann/taglets/1.0/taglets-1.0.jar" ),
-                       new File( getBasedir(), "target/local-repo/org/tullmann/taglets/1.0/taglets-1.0.jar" ) );
+        sourceDir = new File( getBasedir(), "src/test/resources/unit/stylesheetfile-test/artifact-stylesheetfile" );
+        assertTrue( sourceDir.exists() );
+        FileUtils.copyDirectoryStructure( sourceDir, localRepo );
 
-        FileUtils
-            .copyFile(
-                       new File( getBasedir(),
-                                 "src/test/resources/unit/taglet-test/artifact-taglet/org/tullmann/taglets/1.0/taglets-1.0.pom" ),
-                       new File( getBasedir(), "target/local-repo/org/tullmann/taglets/1.0/taglets-1.0.pom" ) );
+        // ----------------------------------------------------------------------
+        // helpfile-test
+        // ----------------------------------------------------------------------
+
+        sourceDir = new File( getBasedir(), "src/test/resources/unit/helpfile-test/artifact-helpfile" );
+        assertTrue( sourceDir.exists() );
+        FileUtils.copyDirectoryStructure( sourceDir, localRepo );
+
+        // Remove SCM files
+        List files =
+            FileUtils.getFileAndDirectoryNames( localRepo, FileUtils.getDefaultExcludesAsString(), null, true,
+                                                true, true, true );
+        for ( Iterator it = files.iterator(); it.hasNext(); )
+        {
+            File file = new File( it.next().toString() );
+
+            if ( file.isDirectory() )
+            {
+                FileUtils.deleteDirectory( file );
+            }
+            else
+            {
+                file.delete();
+            }
+        }
+
+        TEST_REPO_CREATED = true;
     }
 
     /**
@@ -333,6 +330,10 @@ public class JavadocReportTest
         generatedFile = new File( getBasedir(),
                                   "target/test/unit/docfiles-test/target/site/apidocs/doc-files/excluded-dir2" );
         assertTrue( !FileUtils.fileExists( generatedFile.getAbsolutePath() ) );
+
+        testPom = new File( getBasedir(), "src/test/resources/unit/docfiles-with-java-test/docfiles-with-java-test-plugin-config.xml" );
+        mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+        mojo.execute();
     }
 
     /**
@@ -409,6 +410,29 @@ public class JavadocReportTest
         generatedFile = new File( getBasedir(),
                                   "target/test/unit/custom-configuration/target/site/apidocs/custom/configuration/exclude2/Exclude2App.html" );
         assertTrue( !FileUtils.fileExists( generatedFile.getAbsolutePath() ) );
+
+        File options =
+            new File( getBasedir(), "target/test/unit/custom-configuration/target/site/apidocs/options" );
+        assertTrue( options.isFile() );
+        String contentOptions = null;
+        Reader reader = null;
+        try
+        {
+            reader = ReaderFactory.newPlatformReader( options );
+            contentOptions = IOUtil.toString( reader );
+        }
+        finally
+        {
+            IOUtil.close( reader );
+        }
+
+        assertTrue( contentOptions != null );
+        if ( JavadocUtilTest.isWebSiteOnline( null, getContainer().getLogger(), "http://java.sun.com" ) )
+        {
+            assertTrue( contentOptions.indexOf( "-link" ) != -1 );
+            assertTrue( contentOptions.indexOf( "http://java.sun.com/j2se/" ) != -1
+                || contentOptions.indexOf( "http://java.sun.com/javase/" ) != -1 );
+        }
     }
 
     /**
@@ -610,8 +634,8 @@ public class JavadocReportTest
         if ( !SystemUtils.isJavaVersionAtLeast( 1.5f ) )
         {
             getContainer().getLogger().warn(
-                                             "JDK 5.0 or more is required to run javadoc for "
-                                                 + "'org.apache.maven.plugin.javadoc.JavadocReportTest#testJdk5()'." );
+                                             "JDK 5.0 or more is required to run javadoc for '"
+                                                 + getClass().getName() + "#" + getName() + "()'." );
             return;
         }
 
@@ -844,8 +868,8 @@ public class JavadocReportTest
         if ( !SystemUtils.isJavaVersionAtLeast( 1.6f ) )
         {
             getContainer().getLogger().warn(
-                                             "JDK 6.0 or more is required to run javadoc for "
-                                                 + "'org.apache.maven.plugin.javadoc.JavadocReportTest#testJdk6()'." );
+                                             "JDK 6.0 or more is required to run javadoc for '"
+                                                 + getClass().getName() + "#" + getName() + "()'." );
             return;
         }
 
@@ -881,19 +905,137 @@ public class JavadocReportTest
     public void testProxy()
         throws Exception
     {
+        Settings settings = new Settings();
+        Proxy proxy = new Proxy();
+
+        //dummy proxy
+        proxy.setActive( true );
+        proxy.setHost( "127.0.0.1" );
+        proxy.setPort( 80 );
+        proxy.setProtocol( "http" );
+        proxy.setUsername( "toto" );
+        proxy.setPassword( "toto" );
+        proxy.setNonProxyHosts( "www.google.com|*.somewhere.com" );
+        settings.addProxy( proxy );
+
         File testPom = new File( getBasedir(), "src/test/resources/unit/proxy-test/proxy-test-plugin-config.xml" );
         JavadocReport mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+        assertNotNull( mojo );
+        setVariableValueToObject( mojo, "settings", settings );
         mojo.execute();
 
         File commandLine = new File( getBasedir(), "target/test/unit/proxy-test/target/site/apidocs/javadoc." + ( SystemUtils.IS_OS_WINDOWS ? "bat" : "sh" ) );
         assertTrue( FileUtils.fileExists( commandLine.getAbsolutePath() ) );
         String readed = readFile( commandLine );
         assertTrue( readed.indexOf( "-J-Dhttp.proxySet=true" ) != -1 );
-        assertTrue( readed.indexOf( "-J-Dhttp.proxyHost=http://localhost" ) != -1 );
+        assertTrue( readed.indexOf( "-J-Dhttp.proxyHost=127.0.0.1" ) != -1 );
         assertTrue( readed.indexOf( "-J-Dhttp.proxyPort=80" ) != -1 );
         assertTrue( readed.indexOf( "-J-Dhttp.proxyUser=\\\"toto\\\"" ) != -1 );
         assertTrue( readed.indexOf( "-J-Dhttp.proxyPassword=\\\"toto\\\"" ) != -1 );
         assertTrue( readed.indexOf( "-J-Dhttp.nonProxyHosts=\\\"www.google.com|*.somewhere.com\\\"" ) != -1 );
+
+        File options = new File( getBasedir(), "target/test/unit/proxy-test/target/site/apidocs/options" );
+        assertTrue( FileUtils.fileExists( options.getAbsolutePath() ) );
+        String optionsContent = readFile( options );
+        // NO -link http://java.sun.com/j2se/1.5.0/docs/api/package-list
+        if ( JavadocUtilTest.isWebSiteOnline( null, getContainer().getLogger(), "http://java.sun.com" ) )
+        {
+            assertTrue( optionsContent.indexOf( "-link" ) == -1 );
+        }
+
+        // real proxy
+        ProxyServer proxyServer = null;
+        AuthAsyncProxyServlet proxyServlet = null;
+        try
+        {
+            proxyServlet = new AuthAsyncProxyServlet();
+            proxyServer = new ProxyServer( proxyServlet );
+            proxyServer.start();
+
+            settings = new Settings();
+            proxy = new Proxy();
+            proxy.setActive( true );
+            proxy.setHost( proxyServer.getHostName() );
+            proxy.setPort( proxyServer.getPort() );
+            proxy.setProtocol( "http" );
+            settings.addProxy( proxy );
+
+            mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+            setVariableValueToObject( mojo, "settings", settings );
+            mojo.execute();
+            readed = readFile( commandLine );
+            assertTrue( readed.indexOf( "-J-Dhttp.proxySet=true" ) != -1 );
+            assertTrue( readed.indexOf( "-J-Dhttp.proxyHost=" + proxyServer.getHostName() ) != -1 );
+            assertTrue( readed.indexOf( "-J-Dhttp.proxyPort=" + proxyServer.getPort() ) != -1 );
+
+            optionsContent = readFile( options );
+            // -link http://java.sun.com/j2se/1.5.0/docs/api/package-list
+            if ( JavadocUtilTest.isWebSiteOnline( null, getContainer().getLogger(), "http://java.sun.com" ) )
+            {
+                assertTrue( optionsContent.indexOf( "-link" ) != -1 );
+            }
+            assertTrue( true );
+        }
+        catch ( Exception e )
+        {
+            assertTrue( false );
+        }
+        finally
+        {
+            if ( proxyServer != null )
+            {
+                proxyServer.stop();
+            }
+        }
+
+        // auth proxy
+        Map authentications = new HashMap();
+        authentications.put( "foo", "bar" );
+        try
+        {
+            proxyServlet = new AuthAsyncProxyServlet();
+            proxyServer = new ProxyServer( proxyServlet );
+            proxyServer.start();
+
+            settings = new Settings();
+            proxy = new Proxy();
+            proxy.setActive( true );
+            proxy.setHost( proxyServer.getHostName() );
+            proxy.setPort( proxyServer.getPort() );
+            proxy.setProtocol( "http" );
+            proxy.setUsername( "foo" );
+            proxy.setPassword( "bar" );
+            settings.addProxy( proxy );
+
+            mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+            setVariableValueToObject( mojo, "settings", settings );
+            mojo.execute();
+            readed = readFile( commandLine );
+            assertTrue( readed.indexOf( "-J-Dhttp.proxySet=true" ) != -1 );
+            assertTrue( readed.indexOf( "-J-Dhttp.proxyHost=" + proxyServer.getHostName() ) != -1 );
+            assertTrue( readed.indexOf( "-J-Dhttp.proxyPort=" + proxyServer.getPort() ) != -1 );
+            assertTrue( readed.indexOf( "-J-Dhttp.proxyUser=\\\"foo\\\"" ) != -1 );
+            assertTrue( readed.indexOf( "-J-Dhttp.proxyPassword=\\\"bar\\\"" ) != -1 );
+
+            optionsContent = readFile( options );
+            // -link http://java.sun.com/j2se/1.5.0/docs/api/package-list
+            if ( JavadocUtilTest.isWebSiteOnline( null, getContainer().getLogger(), "http://java.sun.com" ) )
+            {
+                assertTrue( optionsContent.indexOf( "-link" ) != -1 );
+            }
+            assertTrue( true );
+        }
+        catch ( Exception e )
+        {
+            assertTrue( false );
+        }
+        finally
+        {
+            if ( proxyServer != null )
+            {
+                proxyServer.stop();
+            }
+        }
     }
 
     /**
@@ -904,8 +1046,10 @@ public class JavadocReportTest
     public void testValidateOptions()
         throws Exception
     {
-        File testPom = new File( getBasedir(),
-                                 "src/test/resources/unit/validate-options-test/wrong-encoding-test-plugin-config.xml" );
+        // encoding
+        File testPom =
+            new File( getBasedir(),
+                      "src/test/resources/unit/validate-options-test/wrong-encoding-test-plugin-config.xml" );
         JavadocReport mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
         try
         {
@@ -914,11 +1058,62 @@ public class JavadocReportTest
         }
         catch ( MojoExecutionException e )
         {
-            assertTrue( "Not wrong encoding catch", e.getMessage().indexOf( "Encoding not supported" ) != -1 );
+            assertTrue( "Not wrong encoding catch",
+                        e.getMessage().indexOf( "Unsupported option <encoding/>" ) != -1 );
+        }
+        testPom =
+            new File( getBasedir(),
+                      "src/test/resources/unit/validate-options-test/wrong-docencoding-test-plugin-config.xml" );
+        mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+        try
+        {
+            mojo.execute();
+            assertTrue( "Not wrong docencoding catch", false );
+        }
+        catch ( MojoExecutionException e )
+        {
+            assertTrue( "Not wrong docencoding catch", e.getMessage()
+                                                        .indexOf( "Unsupported option <docencoding/>" ) != -1 );
+        }
+        testPom =
+            new File( getBasedir(),
+                      "src/test/resources/unit/validate-options-test/wrong-charset-test-plugin-config.xml" );
+        mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+        try
+        {
+            mojo.execute();
+            assertTrue( "Not wrong charset catch", false );
+        }
+        catch ( MojoExecutionException e )
+        {
+            assertTrue( "Not wrong charset catch", e.getMessage().indexOf( "Unsupported option <charset/>" ) != -1 );
         }
 
-        testPom = new File( getBasedir(),
-                            "src/test/resources/unit/validate-options-test/conflict-options-test-plugin-config.xml" );
+        // locale
+        testPom =
+            new File( getBasedir(),
+                      "src/test/resources/unit/validate-options-test/wrong-locale-test-plugin-config.xml" );
+        mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+        try
+        {
+            mojo.execute();
+            assertTrue( "Not wrong locale catch", false );
+        }
+        catch ( MojoExecutionException e )
+        {
+            assertTrue( "Not wrong locale catch", e.getMessage().indexOf( "Unsupported option <locale/>" ) != -1 );
+        }
+        testPom =
+            new File( getBasedir(),
+                      "src/test/resources/unit/validate-options-test/wrong-locale-with-variant-test-plugin-config.xml" );
+        mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+        mojo.execute();
+        assertTrue( "Not wrong locale catch", true );
+
+        // conflict options
+        testPom =
+            new File( getBasedir(),
+                      "src/test/resources/unit/validate-options-test/conflict-options-test-plugin-config.xml" );
         mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
         try
         {
@@ -927,7 +1122,8 @@ public class JavadocReportTest
         }
         catch ( MojoExecutionException e )
         {
-            assertTrue( "Not conflict catch", e.getMessage().indexOf( "Option <nohelp/> conflicts with <helpfile/>" ) != -1 );
+            assertTrue( "Not conflict catch", e.getMessage()
+                                               .indexOf( "Option <nohelp/> conflicts with <helpfile/>" ) != -1 );
         }
     }
 
@@ -974,4 +1170,177 @@ public class JavadocReportTest
         assertTrue( options.indexOf( "org.codehaus.plexus.javadoc.PlexusRequirementTaglet" ) != -1 );
         assertTrue( options.indexOf( "org.codehaus.plexus.javadoc.PlexusComponentTaglet" ) != -1 );
     }
+
+    /**
+     * Method to test the <code>&lt;stylesheetfile/&gt;</code> parameter.
+     *
+     * @throws Exception if any
+     */
+    public void testStylesheetfile()
+        throws Exception
+    {
+        File testPom = new File( getBasedir(), "src/test/resources/unit/stylesheetfile-test/pom.xml" );
+
+        JavadocReport mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+        assertNotNull( mojo );
+
+        setVariableValueToObject( mojo, "remoteRepositories", mojo.project.getRemoteArtifactRepositories() );
+
+        File stylesheetfile =
+            new File( getBasedir(), "target/test/unit/stylesheetfile-test/target/site/apidocs/stylesheet.css" );
+
+        File options = new File( getBasedir(), "target/test/unit/stylesheetfile-test/target/site/apidocs/options" );
+
+        // stylesheet == maven OR java
+        setVariableValueToObject( mojo, "stylesheet", "javamaven" );
+
+        try
+        {
+            mojo.execute();
+            assertTrue( false );
+        }
+        catch ( Exception e )
+        {
+            assertTrue( true );
+        }
+
+        // stylesheet == java
+        setVariableValueToObject( mojo, "stylesheet", "java" );
+        mojo.execute();
+
+        String content = readFile( stylesheetfile );
+        assertTrue( content.indexOf( "/* Javadoc style sheet */" ) != -1 );
+
+        String optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-stylesheetfile" ) == -1 );
+
+        // stylesheet == maven
+        setVariableValueToObject( mojo, "stylesheet", "maven" );
+        mojo.execute();
+
+        content = readFile( stylesheetfile );
+        assertTrue( content.indexOf( "/* Javadoc style sheet */" ) != -1
+            && content.indexOf( "Licensed to the Apache Software Foundation (ASF) under one" ) != -1 );
+
+        optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-stylesheetfile" ) != -1 );
+        assertTrue( optionsContent
+                                  .indexOf( "'" + stylesheetfile.getAbsolutePath().replaceAll( "\\\\", "/" ) + "'" ) != -1 );
+
+        // stylesheetfile defined as a project resource
+        setVariableValueToObject( mojo, "stylesheet", null );
+        setVariableValueToObject( mojo, "stylesheetfile", "com/mycompany/app/javadoc/css/stylesheet.css" );
+        mojo.execute();
+
+        content = readFile( stylesheetfile );
+        assertTrue( content.indexOf( "/* Custom Javadoc style sheet in project */" ) != -1 );
+
+        optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-stylesheetfile" ) != -1 );
+        File stylesheetResource =
+            new File( getBasedir(),
+                      "src/test/resources/unit/stylesheetfile-test/src/main/resources/com/mycompany/app/javadoc/css/stylesheet.css" );
+        assertTrue( optionsContent.indexOf( "'" + stylesheetResource.getAbsolutePath().replaceAll( "\\\\", "/" )
+            + "'" ) != -1 );
+
+        // stylesheetfile defined in a javadoc plugin dependency
+        setVariableValueToObject( mojo, "stylesheetfile", "com/mycompany/app/javadoc/css2/stylesheet.css" );
+        mojo.execute();
+
+        content = readFile( stylesheetfile );
+        assertTrue( content.indexOf( "/* Custom Javadoc style sheet in artefact */" ) != -1 );
+
+        optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-stylesheetfile" ) != -1 );
+        assertTrue( optionsContent
+                                  .indexOf( "'" + stylesheetfile.getAbsolutePath().replaceAll( "\\\\", "/" ) + "'" ) != -1 );
+
+        // stylesheetfile defined as file
+        File css =
+            new File( getBasedir(),
+                      "src/test/resources/unit/stylesheetfile-test/src/main/resources/com/mycompany/app/javadoc/css3/stylesheet.css" );
+        setVariableValueToObject( mojo, "stylesheetfile", css.getAbsolutePath() );
+        mojo.execute();
+
+        content = readFile( stylesheetfile );
+        assertTrue( content.indexOf( "/* Custom Javadoc style sheet as file */" ) != -1 );
+
+        optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-stylesheetfile" ) != -1 );
+        stylesheetResource =
+            new File( getBasedir(),
+                      "src/test/resources/unit/stylesheetfile-test/src/main/resources/com/mycompany/app/javadoc/css3/stylesheet.css" );
+        assertTrue( optionsContent.indexOf( "'" + stylesheetResource.getAbsolutePath().replaceAll( "\\\\", "/" )
+            + "'" ) != -1 );
+    }
+
+    /**
+     * Method to test the <code>&lt;helpfile/&gt;</code> parameter.
+     *
+     * @throws Exception if any
+     */
+    public void testHelpfile()
+        throws Exception
+    {
+        File testPom = new File( getBasedir(), "src/test/resources/unit/helpfile-test/pom.xml" );
+
+        JavadocReport mojo = (JavadocReport) lookupMojo( "javadoc", testPom );
+        assertNotNull( mojo );
+
+        setVariableValueToObject( mojo, "remoteRepositories", mojo.project.getRemoteArtifactRepositories() );
+
+        File helpfile =
+            new File( getBasedir(), "target/test/unit/helpfile-test/target/site/apidocs/help-doc.html" );
+
+        File options = new File( getBasedir(), "target/test/unit/helpfile-test/target/site/apidocs/options" );
+
+        // helpfile by default
+        mojo.execute();
+
+        String content = readFile( helpfile );
+        assertTrue( content.indexOf( "<!-- Generated by javadoc" ) != -1 );
+
+        String optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-helpfile" ) == -1 );
+
+        // helpfile defined in a javadoc plugin dependency
+        setVariableValueToObject( mojo, "helpfile", "com/mycompany/app/javadoc/helpfile/help-doc.html" );
+        mojo.execute();
+
+        content = readFile( helpfile );
+        assertTrue( content.indexOf( "<!--  Help file from artefact -->" ) != -1 );
+
+        optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-helpfile" ) != -1 );
+        File help = new File( getBasedir(), "target/test/unit/helpfile-test/target/site/apidocs/help-doc.html" );
+        assertTrue( optionsContent.indexOf( "'" + help.getAbsolutePath().replaceAll( "\\\\", "/" ) + "'" ) != -1 );
+
+        // helpfile defined as a project resource
+        setVariableValueToObject( mojo, "helpfile", "com/mycompany/app/javadoc/helpfile2/help-doc.html" );
+        mojo.execute();
+
+        content = readFile( helpfile );
+        assertTrue( content.indexOf( "<!--  Help file from file -->" ) != -1 );
+
+        optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-helpfile" ) != -1 );
+        help =
+            new File( getBasedir(),
+                      "src/test/resources/unit/helpfile-test/src/main/resources/com/mycompany/app/javadoc/helpfile2/help-doc.html" );
+        assertTrue( optionsContent.indexOf( "'" + help.getAbsolutePath().replaceAll( "\\\\", "/" ) + "'" ) != -1 );
+
+        // helpfile defined as file
+        help =
+            new File( getBasedir(),
+                      "src/test/resources/unit/helpfile-test/src/main/resources/com/mycompany/app/javadoc/helpfile2/help-doc.html" );
+        setVariableValueToObject( mojo, "helpfile", help.getAbsolutePath() );
+        mojo.execute();
+
+        content = readFile( helpfile );
+        assertTrue( content.indexOf( "<!--  Help file from file -->" ) != -1 );
+
+        optionsContent = readFile( options );
+        assertTrue( optionsContent.indexOf( "-helpfile" ) != -1 );
+        assertTrue( optionsContent.indexOf( "'" + help.getAbsolutePath().replaceAll( "\\\\", "/" ) + "'" ) != -1 );
+    }
 }
